import os
import inspect
from jinja2 import ChoiceLoader, FileSystemLoader, PackageLoader
from run import (Module, FindModule,
                 NullTask, RenderTask, DerivedTask,
                 AttributeVar, FindVar)

class ManageModule(Module):

    # Public

    def __init__(self):
        self._update_pythonpath()

    # Meta

    meta_strict = False

    # Attributes

    author = '<author>'
    author_email = '<author_email>'
    caution = 'DO NOT CHANGE THIS FILE. SOURCE IS IN "_sources" DIRECTORY.'
    classifiers = []
    data_files = []
    description = '<description>'
    development_requires = []
    entry_points = {}
    install_requires = []
    license = '<license>'
    name = '<name>'
    platforms = []
    pypi_password_secure = '<pypi_password_secure>'
    tests_require = []
    test_suite = '<test_suite>'

    # Modules

    docs = FindModule(names=['docs'])
    tests = FindModule(names=['tests'])

    # Tasks

    build = NullTask(
        meta_require=[
            'build_gitignore',
            'build_travis',
            'build_manifest',
            'build_readme',
            'build_requirements',
            'build_setup',
            'docs.build',
        ],
    )

    build_gitignore = DerivedTask(
        task='render',
        source='.gitignore',
        target='.gitignore',
    )

    build_travis = DerivedTask(
        task='render',
        source='.travis.yml',
        target='.travis.yml',
    )

    build_manifest = DerivedTask(
        task='render',
        source='MANIFEST.in',
        target='MANIFEST.in',
    )

    build_readme = DerivedTask(
        task='render',
        source='README.rst',
        target='README.rst',
    )

    build_requirements = DerivedTask(
        task='render',
        source='requirements.txt',
        target='requirements.txt',
    )

    build_setup = DerivedTask(
        task='render',
        source='setup.py',
        target='setup.py',
    )

    render = RenderTask(
        trim_blocks=True,
        lstrip_blocks=True,
        loader=ChoiceLoader([
            FileSystemLoader('_sources'),
            PackageLoader('packgram', '_sources'),
        ]),
    )

    # Vars

    long_description = FindVar(
        filepathes=['README.rst'],
        getfirst=True,
    )

    github_user = AttributeVar(
        attribute='author',
    )

    maintainer = AttributeVar(
        attribute='author',
    )

    maintainer_email = AttributeVar(
        attribute='author_email',
    )

    pypi_name = AttributeVar(
        attribute='name',
    )

    pypi_user = AttributeVar(
        attribute='author',
    )

    version = FindVar(
        mode='objects',
        filepathes=['*/version.py'],
        filters=[{'objname': 'version'}],
        getfirst=True,
    )

    # Protected

    def _update_pythonpath(self):
        os.environ['PYTHONPATH'] = ':'.join([
            os.environ.get('PYTHONPATH', ''),
            os.path.abspath(os.path.dirname(inspect.getfile(type(self))))])
